import jwt
from datetime import datetime
from django.conf import settings


class JwtHelper(object):
    @staticmethod
    def encode(payload:dict)->str:
        data={
            'iat': datetime.utcnow(),
            'exp': datetime.utcnow()+settings.SHA_ACCOUNTS.get('JWT_ACCESS_TOKEN_EXP'),
        }
        for key,value in payload.items():
            data[key]=value
        token=jwt.encode(payload,settings.SECRET_KEY,algorithm='HS256')
        return token

    @staticmethod
    def decode(token:str)->dict:
        payload=jwt.decode(token,settings.SECRET_KEY,algorithms=['HS256'])
        return payload


from django.test import TestCase
from ..models import User, Permission


class UserModelTestCase(TestCase):
    def setUp(self):
        pass

    def test_create_normal_user(self):
        normal_user = User.objects.create(
            username='normal', email='normal@example.com', password='normal')
        self.assertIsNotNone(normal_user)
        self.assertIsNotNone(normal_user.permissions.all())
        self.assertIsNotNone(normal_user.profile)

    def test_create_superuser(self):
        superuser = User.objects.create_superuser(
            username='superuser', email='superuser@example.com', password='superuser')
        self.assertIsNotNone(superuser)
        self.assertIsNotNone(superuser.permissions.all())
        self.assertIsNotNone(superuser.profile)



from rest_framework import serializers
# from ..models.profile_models import Profile
from ..models import get_user_profile_model

Profile = get_user_profile_model()


class ProfileSerializer(serializers.ModelSerializer):
    class Meta:
        model = Profile
        exclude = ['id', 'user']



from django.contrib.auth.models import BaseUserManager
from django.db import IntegrityError
from djrest_wrapper.exceptions import DuplicateModelExp


class UserManager(BaseUserManager):
    def create(self, username, email, password, * args, **kwargs):
        try:
            new_user = super().create(username=username, email=email, *args, **kwargs)
            new_user.set_password(password)
            new_user.save(using=self._db)
            return new_user
        except IntegrityError as e:
            raise DuplicateModelExp('user already exists')

    def create_superuser(self, username, email, password):
        user = self.create(
            username=username, email=email, password=password)
        user.is_superuser = True
        user.save(using=self._db)
        return user



from django.test import TestCase
from ..serializers.user_serializers import UserSignUpRequest, UserSignUpResponse
from ..models.permission_models import Permission
from ..models import get_user_profile_model


class UserSerializerTestCase(TestCase):
    def setUp(self):
        pass

    def test_usersignuprequest_serializer(self):
        new_data = {
            'username': 'ali',
            'email': 'ali@examplle.com',
            'password': '3ecret'
        }
        serializer = UserSignUpRequest(data=new_data)
        serializer.is_valid(raise_exception=True)
        new_user = serializer.save()
        self.assertIsNotNone(new_user)
        self.assertIsInstance(new_user.profile, get_user_profile_model())
        for permission in new_user.permissions.all():
            self.assertIsInstance(permission, Permission)
        return new_user

    def test_usersignupresponse_serializer(self):
        user = self.test_usersignuprequest_serializer()
        serializer = UserSignUpResponse(user)
        self.assertIsNotNone(serializer)



from rest_framework.decorators import action
from rest_framework.status import HTTP_201_CREATED
from rest_framework.response import Response
from djrest_wrapper.interfaces import BaseViewSet
from djrest_wrapper.decorators import serializer_validation
from ..models.user_models import User
from ..signals import user_logged_in, user_logged_out
from ..serializers.user_serializers import (
    UserSignUpRequest, UserSignUpResponse, UserSignInRequest,
    UserSignInResponse, UserListResponse, UserUpdateRequest,
    UserUpdateResponse, UserChangePasswordRequest,)
from ..permissions import IsAuthenticatedAndOwner, IsAdmin


class UserViewSet(BaseViewSet):
    queryset = User.objects.all()
    page_result_key = 'users'
    serializer_action_classes = {
        'create': {
            'req': UserSignUpRequest,
            'res': UserSignUpResponse,
        },
        'retrieve': {
            'res': UserSignUpResponse,
        },
        'signin': {
            'req': UserSignInRequest,
            'res': UserSignInResponse,
        },
        'list': {
            'res': UserListResponse,
        },
        'update': {
            'req': UserUpdateRequest,
            'res': UserUpdateResponse,
        },
        'partial_update': {
            'req': UserUpdateRequest,
            'res': UserUpdateResponse,
        },
        'change_password': {
            'req': UserChangePasswordRequest,
        }
    }
    permission_action_classes = {
        'retrieve': [IsAuthenticatedAndOwner | IsAdmin],
        'update': [IsAuthenticatedAndOwner | IsAdmin],
        'partial_update': [IsAuthenticatedAndOwner | IsAdmin],
        'list': [IsAdmin],
        'change_password': [IsAuthenticatedAndOwner, ],
    }

    @serializer_validation
    def create(self, request, *args, **kwargs):
        reqser = self.get_serializer(data=request.data)
        reqser.is_valid(raise_exception=True)
        model = self.perform_create(reqser)
        model.access_token = user_logged_in.send(sender=User, user=model)[0][1]
        resser = self.get_serializer_response()(model)
        return Response(
            data={model.__class__.__name__.lower(): resser.data},
            status=HTTP_201_CREATED)

    @action(detail=False, methods=['POST'],
            url_name='signin', url_path='signin')
    @serializer_validation
    def signin(self, request, *args, **kwargs):
        reqser = self.get_serializer(data=request.data)
        reqser.is_valid(raise_exception=True)
        user = reqser.login()
        user.access_token = user_logged_in.send(sender=User, user=user)[0][1]
        resser = self.get_serializer_response()(user)
        return Response(data={user.__class__.__name__.lower(): resser.data})

    @action(detail=False, methods=['GET'],
            url_name='signout', url_path='signout')
    def signout(self, request, *args, **kwargs):
        user_logged_out.send(sender=User, request=request)
        return Response(data={})

    @action(detail=False, methods=['POST'],
            url_name='change-password', url_path='change-password')
    @serializer_validation
    def change_password(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        is_changed = serializer.change_password(id=request.user.id)
        if is_changed:
            user_logged_out.send(sender=User, request=request)
            return Response(data={})



from django.apps import AppConfig


class ExamplesConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'examples'

    def ready(self):
        import examples.signals



from pathlib import Path
from os import getenv
from datetime import timedelta

BASE_DIR = Path(__file__).resolve().parent.parent

SECRET_KEY = getenv('DJANGO_SECRET_KEY')

DEBUG = getenv('DJANGO_DEBUG', True)

ALLOWED_HOSTS = getenv('DJANGO_ALLOWED_HOSTS', 'localhost').split(',')

DJANGO_DEFAULT_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
]

DEBUG_APPS = [
    'django.contrib.staticfiles',
    'drf_yasg',
]

THIRD_PARTIES = [
    'rest_framework',
]

LOCAL_APPS = [
    'sha_accounts',
    'examples'
]

INSTALLED_APPS = DJANGO_DEFAULT_APPS+THIRD_PARTIES+LOCAL_APPS
if DEBUG:
    INSTALLED_APPS += DEBUG_APPS


MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'configs.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'configs.wsgi.application'

if DEBUG:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': BASE_DIR / 'db.sqlite3',
        }
    }
    CACHES = {
        'default': {
            'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        },
        'token-cache': {
            'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        },
    }
else:
    # Set a database connection and cache
    pass

# Set your customized REST_FRAMEWORK settings
REST_FRAMEWORK = {
    'EXCEPTION_HANDLER': 'djrest_wrapper.exceptions.handler.exception_handler',
    'DEFAULT_RENDERER_CLASSES': ['djrest_wrapper.renderers.defaultjson.DefaultJsonRenderer'],
    'DEFAULT_PAGINATION_CLASS': 'djrest_wrapper.paginations.default.DefaultPagination',
    'DEFAULT_AUTHENTICATION_CLASSES': [
        'sha_accounts.backends.authentications.JwtAuthentication',
         ]}

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

STATIC_URL = '/static/'

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'


AUTH_USER_MODEL = 'sha_accounts.User'

SHA_ACCOUNTS = {
    'DEFAULT_ACTIVATION': True,
    'AUTH_USER_PROFILE_MODEL': 'examples.ExampleProfile',
    'JWT_ACCESS_TOKEN_EXP': timedelta(days=1),
    'JWT_USER_ENCODED_FIELDS': ['id'],
    'JWT_AUTH_RAELM': 'sample_raelm'
}



from django.apps import AppConfig


class ShaAccountsConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'sha_accounts'

    def ready(self):
        import sha_accounts.signals



from django import dispatch

user_logged_in = dispatch.Signal(providing_args=["user"])
user_logged_out = dispatch.Signal(providing_args=["user"])
user_login_failed = dispatch.Signal(providing_args=["user"])



from django.urls import reverse
from django.conf import settings
from rest_framework import status
from rest_framework.test import APITestCase
from djrest_wrapper.exceptions.apis import errors
from ..models.user_models import User
from ..models import get_user_profile_model

class UserViewSetTestCase(APITestCase):
    def setUp(self):
        pass

    def test_create_user_view(self):
        url = reverse('user-list')
        data = {
            'username': 'testuser',
            'email': 'test@example.com',
            'password': 'test'
        }
        response = self.client.post(path=url, data=data, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.json().get('err'), False)
        self.assertEqual(response.json().get(
            'err_code'), errors.ERR_SUCCESSFUL)
        self.assertIsNotNone(response.json().get(
            'data').get('user').get('access_token'))

    def test_signin_user_view(self):
        self.test_create_user_view()
        url = reverse('user-signin')
        data = {
            'username': 'testuser',
            'password': 'test'
        }
        response = self.client.post(path=url, data=data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.json().get('err'), False)
        self.assertEqual(response.json().get(
            'err_code'), errors.ERR_SUCCESSFUL)
        self.assertIsNotNone(response.json().get(
            'data').get('user').get('access_token'))
        return response.json().get('data').get('user')

    def test_retrieve_user_profile_view_succesful(self):
        user = self.test_signin_user_view()
        self.client.credentials(
            HTTP_AUTHORIZATION=f'{settings.SHA_ACCOUNTS.get("JWT_AUTH_RAELM")} {user.get("access_token")}')
        url = reverse('user-detail', args={user.get('id')})
        response = self.client.get(path=url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.json().get('err'), False)
        self.assertEqual(response.json().get(
            'err_code'), errors.ERR_SUCCESSFUL)
        self.assertIsNotNone(response.json().get('data').get('user'))

    def test_retrieve_user_profile_view_failed(self):
        user = self.test_signin_user_view()
        url = reverse('user-detail', args={user.get('id')})
        response = self.client.get(path=url)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertEqual(response.json().get('err'), True)
        self.assertEqual(response.json().get(
            'err_code'), errors.ERR_PERMISSION_DENIED)

    def test_retrieve_user_profile_admin_view(self):
        superuser = User.objects.create_superuser(
            username='superuser',
            email='superuser@example.com',
            password='superuser')
        user = User.objects.create(
            username='testuser',
            email='testuser@example.com',
            password='testuser')
        url = reverse('user-signin')
        data = {
            'username': superuser.username,
            'password': 'superuser'
        }
        response = self.client.post(path=url, data=data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.json().get('err'), False)
        self.assertEqual(response.json().get(
            'err_code'), errors.ERR_SUCCESSFUL)
        self.assertIsNotNone(response.json().get(
            'data').get('user').get('access_token'))
        self.client.credentials(
            HTTP_AUTHORIZATION=f'{settings.SHA_ACCOUNTS.get("JWT_AUTH_RAELM")} {response.json().get("data").get("user").get("access_token")}')
        url = reverse('user-detail', args={user.id})
        response = self.client.get(path=url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.json().get('err'), False)
        self.assertEqual(response.json().get(
            'err_code'), errors.ERR_SUCCESSFUL)
        self.assertIsNotNone(response.json().get('data').get('user'))

    def test_user_list_admin_view(self):
        superuser = User.objects.create_superuser(
            username='superuser',
            email='superuser@example.com',
            password='superuser')
        url = reverse('user-signin')
        data = {
            'username': superuser.username,
            'password': 'superuser'
        }
        response = self.client.post(path=url, data=data, format='json')

        for i in range(12):
            User.objects.create(
                username=f'test{i}',
                email=f'test{i}@example.com',
                password=f'test{i}')
        url = reverse('user-list')
        self.client.credentials(
            HTTP_AUTHORIZATION=f'{settings.SHA_ACCOUNTS.get("JWT_AUTH_RAELM")} {response.json().get("data").get("user").get("access_token")}')
        response = self.client.get(path=url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.json().get('err'), False)
        self.assertEqual(response.json().get(
            'err_code'), errors.ERR_SUCCESSFUL)
        self.assertIsNotNone(response.json().get('data').get('page'))
        self.assertIsNotNone(response.json().get('data').get('users'))

    def test_user_update_profile_put_view(self):
        user = User.objects.create(
            username=f'test',
            email=f'test@example.com',
            password=f'test')
        access_token = self._login('test', 'test').get(
            'data').get('user').get('access_token')
        self.client.credentials(
            HTTP_AUTHORIZATION=f'{settings.SHA_ACCOUNTS.get("JWT_AUTH_RAELM")} {access_token}')
        url = reverse('user-detail', args={user.id})
        data = {
            'profile': {
                'first_name': 'ali',
            },
        }
        response = self.client.put(path=url, data=data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.json().get('err'), False)
        self.assertEqual(response.json().get(
            'err_code'), errors.ERR_SUCCESSFUL)
        self.assertEqual(response.json().get('data').get('user').get(
            'profile').get('first_name'), data.get('profile').get('first_name'))

    def test_user_update_profile_patch_view(self):
        user = User.objects.create(
            username=f'test',
            email=f'test@example.com',
            password=f'test')
        access_token = self._login('test', 'test').get(
            'data').get('user').get('access_token')
        self.client.credentials(
            HTTP_AUTHORIZATION=f'{settings.SHA_ACCOUNTS.get("JWT_AUTH_RAELM")} {access_token}')
        url = reverse('user-detail', args={user.id})
        data = {
            'profile': {
                'first_name': 'ali',
            },
        }
        response = self.client.patch(path=url, data=data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.json().get('err'), False)
        self.assertEqual(response.json().get(
            'err_code'), errors.ERR_SUCCESSFUL)
        self.assertEqual(response.json().get('data').get('user').get(
            'profile').get('first_name'), data.get('profile').get('first_name'))

    def test_delete_user_view(self):
        user = User.objects.create(
            username=f'test',
            email=f'test@example.com',
            password=f'test')
        access_token = self._login('test', 'test').get(
            'data').get('user').get('access_token')
        self.client.credentials(
            HTTP_AUTHORIZATION=f'{settings.SHA_ACCOUNTS.get("JWT_AUTH_RAELM")} {access_token}')
        url = reverse('user-detail', args={user.id})
        response = self.client.delete(path=url)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        try:
            deleting_user = User.objects.get(username=f'test')
            deleting_user_profile = get_user_profile_model().objects.get(user=user)
            self.assertIsNone(deleting_user)
            self.assertIsNone(deleting_user_profile)
        except User.DoesNotExist:
            self.assertTrue(True)

    def test_user_signout_view(self):
        user = User.objects.create(
            username=f'test',
            email=f'test@example.com',
            password=f'test')
        access_token = self._login('test', 'test').get(
            'data').get('user').get('access_token')
        self.client.credentials(
            HTTP_AUTHORIZATION=f'{settings.SHA_ACCOUNTS.get("JWT_AUTH_RAELM")} {access_token}')
        url = reverse('user-signout')
        response = self.client.get(path=url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        url = reverse('user-detail', args={user.id})
        response = self.client.get(path=url)
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_change_password_view(self):
        user = User.objects.create(
            username=f'test',
            email=f'test@example.com',
            password=f'test')
        access_token = self._login('test', 'test').get(
            'data').get('user').get('access_token')
        self.client.credentials(
            HTTP_AUTHORIZATION=f'{settings.SHA_ACCOUNTS.get("JWT_AUTH_RAELM")} {access_token}')
        url = reverse('user-change-password')
        data = {
            'old_password': 'test',
            'new_password': 'test1'
        }
        response = self.client.post(path=url, data=data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.json().get('err'), False)
        self.assertEqual(response.json().get(
            'err_code'), errors.ERR_SUCCESSFUL)

    def _login(self, username, password):
        data = {
            'username': username,
            'password': password
        }
        url = reverse('user-signin')
        response = self.client.post(path=url, data=data, format='json')
        return response.json()



from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.hashers import make_password
from djrest_wrapper.exceptions.apis.exceptions import DoesNotExistsExp
from rest_framework import serializers
from drf_writable_nested.serializers import WritableNestedModelSerializer
from ..models.user_models import User
from .profile_serializers import ProfileSerializer


class UserSignUpRequest(serializers.ModelSerializer):
    username = serializers.CharField(required=True)
    email = serializers.EmailField(required=True)
    password = serializers.CharField(
        write_only=True,
        required=True,
    )

    class Meta:
        model = User
        fields = ['username', 'email', 'password']


class UserSignUpResponse(WritableNestedModelSerializer):
    profile = ProfileSerializer()
    access_token = serializers.CharField(required=False)

    class Meta:
        model = User
        fields = ['id', 'username', 'email',
                  'is_active', 'is_superuser', 'profile', 'access_token']


class UserSignInRequest(serializers.ModelSerializer):
    username = serializers.CharField(required=True)
    password = serializers.CharField(write_only=True, required=True)

    class Meta:
        model = User
        fields = ['username', 'password']

    def login(self):
        username = self.validated_data.get('username')
        password = self.validated_data.get('password')
        try:
            user = self.Meta.model.objects.get(username=username)
            if user.check_password(password):
                return user
            else:
                raise DoesNotExistsExp(f'Invalid Credentials')
        except ObjectDoesNotExist as e:
            raise DoesNotExistsExp(f'Invalid Credentials')


class UserSignInResponse(UserSignUpResponse):
    class Meta(UserSignUpResponse.Meta):
        pass


class UserListResponse(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ['id', 'username', 'is_active', 'is_superuser']


class UserUpdateRequest(WritableNestedModelSerializer):
    profile = ProfileSerializer()

    class Meta:
        model = User
        fields = ['profile']


class UserUpdateResponse(UserSignUpResponse):
    class Meta(UserSignUpResponse.Meta):
        pass


class UserChangePasswordRequest(serializers.ModelSerializer):
    old_password = serializers.CharField(
        write_only=True,
        required=True,
    )
    new_password = serializers.CharField(
        write_only=True,
        required=True,
    )

    def change_password(self, id):
        old_password = self.validated_data.get('old_password')
        new_password = self.validated_data.get('new_password')
        try:
            user = self.Meta.model.objects.get(id=id)
            if user.check_password(old_password):
                user.set_password(new_password)
                return True
            else:
                raise DoesNotExistsExp(f'Invalid Credentials')
        except ObjectDoesNotExist as e:
            raise DoesNotExistsExp(f'Invalid Credentials')

    class Meta:
        model = User
        fields = ['new_password', 'old_password']



from django.dispatch import receiver
from sha_accounts.signals import user_logged_in
from django.contrib.auth import get_user_model


@receiver(user_logged_in, sender=get_user_model())
def send_activation_mail(sender, user, **kwargs):
    pass



from django.core.cache import caches
from django.conf import settings
from django.dispatch import receiver
from django.db.models.signals import post_save
from ..utils.jwt import JwtHelper
from ..models.user_models import User
from ..models import get_user_profile_model
from .signals import user_logged_in, user_logged_out, user_login_failed


@receiver(post_save, sender=User)
def create_profile(sender, instance: User, created, **kwargs):
    if created:
        get_user_profile_model().objects.create(user=instance)


@receiver(user_logged_in, sender=User)
def set_jwt_token(sender, user, **kwargs):
    user_fields = {}
    for field in settings.SHA_ACCOUNTS.get('JWT_USER_ENCODED_FIELDS'):
        if field == 'id':
            user_fields[field] = str(getattr(user, field, None))
            continue
        user_fields[field] = getattr(user, field, None)
    token = JwtHelper.encode({'user': user_fields})
    caches['token-cache'].set(token, 0)
    return token


@receiver(user_logged_out, sender=User)
def delete_jwt_token(sender, request, **kwrargs):
    token = request.headers.get('Authorization').split(' ')[1]
    caches['token-cache'].delete(token)



from django.conf import settings
from .signals import user_logged_in, user_logged_out, user_login_failed

if settings.AUTH_USER_MODEL == 'sha_accounts.User':
    from .receivers import create_profile



from django.contrib.auth.models import AbstractBaseUser
from django.contrib.auth.validators import UnicodeUsernameValidator
from django.utils.translation import gettext_lazy as _
from django.db import models
from djrest_wrapper.interfaces import BaseModel
from django.conf import settings
from ..managers import UserManager


class AbstractUser(AbstractBaseUser, BaseModel):
    username_validator = UnicodeUsernameValidator()
    last_login = None
    username = models.CharField(
        _('username'),
        max_length=150,
        unique=True,
        help_text=_(
            'Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.'),
        validators=[username_validator],
        error_messages={
            'unique': _("A user with that username already exists."),
        },
    )
    email = models.EmailField(
        verbose_name='Email Address', max_length=254, unique=True)
    is_active = models.BooleanField(
        default=settings.SHA_ACCOUNTS.get('DEFAULT_ACTIVATION', True))
    is_superuser = models.BooleanField(_('superuser status'), default=False)

    USERNAME_FIELD = 'username'
    # email and password are required by default
    REQUIRED_FIELDS = ['email']

    objects = UserManager()

    class Meta:
        verbose_name = 'user'
        verbose_name_plural = 'users'
        abstract = True

    def __str__(self):
        return self.email

    def has_perm(self, perm, obj=None):
        return True

    def has_module_perms(self, app_label):
        return True


class User(AbstractUser):
    permissions = models.ManyToManyField('sha_accounts.Permission')



import jwt
from django.core.cache import caches
from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.models import AnonymousUser
from rest_framework.authentication import BaseAuthentication
from djrest_wrapper.exceptions.apis import AuthenticationFailedExp, DoesNotExistsExp
from django.conf import settings
from ..utils.jwt import JwtHelper


class JwtAuthentication(BaseAuthentication):
    def get_authorization_header(self, request):
        RAELM = settings.SHA_ACCOUNTS.get('JWT_AUTH_RAELM')
        auth_header = request.headers.get('Authorization', None)
        if auth_header != None:
            raelm, token = auth_header.split(' ')
            if raelm == RAELM:
                return token
            else:
                raise AuthenticationFailedExp(
                    f'raelm {raelm} is not a valid raelm')
        else:
            return None

    def get_decoded_jwt_token(self, token):
        token_is_valid = caches['token-cache'].get(token)
        if token_is_valid == 0:
            try:
                return JwtHelper.decode(token)
            except jwt.DecodeError as e:
                raise AuthenticationFailedExp(f'not a valid jwt token')
            except jwt.ExpiredSignatureError as e:
                raise AuthenticationFailedExp(f'not a valid jwt token')
        else:
            raise AuthenticationFailedExp(f'try to authenticate')

    def authenticate(self, request):
        token = self.get_authorization_header(request)
        if token != None:
            payload = self.get_decoded_jwt_token(token)
            try:
                user = get_user_model().objects.get(
                    id=payload.get('user').get('id'))
                if user.is_active == True:
                    return (user, None)
                else:
                    raise AuthenticationFailedExp(
                        f'your account is not active')
            except ObjectDoesNotExist as e:
                raise DoesNotExistsExp(
                    f'User {payload.get("user").get("id")} not found')
        else:
            return (AnonymousUser(), None)



from django.conf import settings
from django.apps import apps as django_apps
from django.core.exceptions import ImproperlyConfigured


def get_user_profile_model():
    try:
        return django_apps.get_model(settings.SHA_ACCOUNTS.get(
                                         'AUTH_USER_PROFILE_MODEL'))
    except ValueError:
        raise ImproperlyConfigured(
            "AUTH_USER_PROFILE_MODEL must be of the form 'app_label.model_name'")
    except LookupError:
        raise ImproperlyConfigured(
            "AUTH_USER_PROFILE_MODEL refers to model '%s' that has not been installed" %
            settings.SHA_ACCOUNTS.get('AUTH_USER_PROFILE_MODEL'))


if settings.AUTH_USER_MODEL == 'sha_accounts.User':
    from .permission_models import Permission
    from .user_models import User



from django.test import TestCase
from ..utils.jwt import JwtHelper


class UtilsTestCase(TestCase):
    def setUp(self):
        pass

    def test_encode_decode_jwt(self):
        p={'user':{'id':1},'user2':2}
        token=JwtHelper.encode(p)
        self.assertIsNotNone(token)
        payload=JwtHelper.decode(token)
        self.assertIsNotNone(payload)
        self.assertEqual(p,payload)


from django.db import models
from djrest_wrapper.interfaces import BaseModel


class ExampleProfile(BaseModel):
    first_name = models.CharField(max_length=50, null=True, blank=True)
    last_name = models.CharField(max_length=50, null=True, blank=True)
    user = models.OneToOneField(
        'sha_accounts.User',
        related_name='profile',
        on_delete=models.CASCADE)



from rest_framework.permissions import BasePermission


class IsAuthenticatedAndOwner(BasePermission):
    def has_permission(self, request, view):
        return request.user.is_authenticated

    def has_object_permission(self, request, view, obj):
        return request.user == obj


class IsAdmin(BasePermission):
    def has_permission(self, request, view):
        return request.user.is_superuser



from django.db import models
from djrest_wrapper.interfaces import BaseModel


class AbstractPermission(BaseModel):
    name = models.CharField(max_length=100)

    class Meta:
        abstract = True
        verbose_name = 'permission'
        verbose_name_plural = 'permissions'


class Permission(AbstractPermission):
    pass
